#-*- coding: utf-8 -*-
import os, datetime, thread2, gobject
path = os.getcwd()

def getBackend(preset, xid):
	backend = None
	if preset == 'autodetect':
		if not backend:
			try:
				import gst, pygst
				backend = GST(gst, xid)
				return backend
			except:
				pass
		if not backend:
			try:
				import vlc
				os.chdir(path)
				backend = VLCPlayer(vlc, xid)
				return backend
			except:
				pass
		if not backend:
			try:
				from win32com.client import Dispatch
				backend = WMPlayer(Dispatch, xid)
				return backend
			except:
				pass
		if not backend:
			try:
				import mpylayer
				backend = MPlayer(mpylayer, xid)
				return backend
			except:
				pass
	else:
		if preset == 'vlc':
			try:
				import vlc
				os.chdir(path)
				backend = VLCPlayer(vlc, xid)
				return backend
			except Exception, e:
				print e
		elif preset == 'mplayer':
			try:
				import mpylayer
				backend = MPlayer(mpylayer, xid)
				return backend
			except:
				pass
		elif preset == 'gstreamer':
			try:
				import gst, pygst
				backend = GST(gst, xid)
				return backend
			except Exception, e:
				print e
		elif preset == 'Windows Media Player':
			try:
				from win32com.client import Dispatch
				backend = WMPlayer(Dispatch, xid)
				return backend
			except:
				pass
		else:
			return None

class WMPlayer:
	def __init__(self, Dispatch, xid):
		print 'using WMPlayer'
		self.mp = Dispatch("WMPlayer.OCX")
		self.playing = False
		
	def load(self, url):
		tune = self.mp.newMedia(url)
		self.mp.currentPlaylist.appendItem(tune)
		
	def play(self):
		self.mp.controls.play()
		self.playing = True
	
	def pause(self):
		self.mp.controls.pause()
		self.playing = False
		
	def unload(self):
		self.mp.currentPlaylist.clear()
		self.mp.controls.stop()
		
	def volume(self, value):
		self.mp.settings.volume = int(value)
		
	def seek(self, pos):
		self.mp.controls.currentPosition = pos
		
	def isPlaying(self):
		return self.playing
		
	def getPos(self):
		import time
		try:
			dur_int = self.mp.currentMedia.duration
			cur_int = self.mp.controls.currentPosition
			if dur_int == 0:
				dur_int = -1
			pos = float(cur_int) / float(dur_int) * 100
			cur = time.strftime('%M:%S', time.gmtime(cur_int))
			tot = time.strftime('%M:%S', time.gmtime(dur_int))
			return [pos, cur, tot]
		except Exception, e:
			return [0, '00:00', '00:00']
			
class GST:
	def __init__(self, gst, xid):
		self.gst = gst
		self.mp = gst.element_factory_make("playbin2", "player")
		fakesink = gst.element_factory_make("fakesink", "fakesink")
		volume = gst.element_factory_make("volume", "volume")
		self.mp.add(volume)
		#self.mp.set_property("volume", int(self.volume/100))
		self.mp.set_property("video-sink", fakesink)
		self.playing = False
		
	def load(self, url):
		self.mp.set_property("uri", url)
		
	def play(self):
		self.mp.set_state(self.gst.STATE_PLAYING)
		self.playing = True
	
	def pause(self):
		self.mp.set_state(self.gst.STATE_PAUSED)
		self.playing = False
		
	def unload(self):
		self.mp.set_state(self.gst.STATE_NULL)
		self.playing = False
		
	def volume(self, value):
		volume = value / 100
		self.mp.set_property("volume", volume)
		
	def seek(self, pos):
		time_format = self.gst.Format(self.gst.FORMAT_TIME)
		self.mp.seek_simple(time_format, self.gst.SEEK_FLAG_FLUSH, pos)
		
	def isPlaying(self):
		return self.playing
		
	def getPos(self):
		try:
			time_format = self.gst.Format(self.gst.FORMAT_TIME)
			dur_int = self.mp.query_duration(time_format, None)[0]
			cur_int = self.mp.query_position(time_format, None)[0]
			if dur_int == 0:
				dur_int = -1
			pos = float(cur_int) / float(dur_int) * float(100)
			cur = self.convert_ns(cur_int)
			tot = self.convert_ns(dur_int)
			return [pos, cur, tot]
		except:
			return [0, '00:00', '00:00']
	
	def convert_ns(self, time_int):
		time_int = time_int / 1000000000
		time_str = ""
		if time_int >= 3600:
			_hours = time_int / 3600
			time_int = time_int - (_hours * 3600)
			time_str = str(_hours) + ":"
		if time_int >= 600:
			_mins = time_int / 60
			time_int = time_int - (_mins * 60)
			time_str = time_str + str(_mins) + ":"
		elif time_int >= 60:
			_mins = time_int / 60
			time_int = time_int - (_mins * 60)
			time_str = time_str + "0" + str(_mins) + ":"
		else:
			time_str = time_str + "00:"
		if time_int > 9:
			time_str = time_str + str(time_int)
		else:
			time_str = time_str + "0" + str(time_int)
		
		return time_str
			
class VLCPlayer:
	def __init__(self, vlc, xid):
		if xid:
			Instance = vlc.Instance()
		else:
			Instance = vlc.Instance(['--vout', 'dummy'])
		self.mp = vlc.MediaPlayer(Instance)
		self.playing = False
		os.chdir(path)
		
	def load(self, url):
		self.mp.set_mrl(url)
		
	def play(self):
		self.mp.play()
		self.playing = True
		
	def pause(self):
		self.mp.pause()
		self.playing = False
		
	def unload(self):
		self.mp.stop()
		self.playing = False
		
	def volume(self, value):
		self.mp.audio_set_volume(int(value))
		
	def seek(self, pos):
		goto = int(self.mp.get_length() * float(pos / 100))
		self.mp.set_time(goto)
	
	def isPlaying(self):
		return self.playing
		
	def getPos(self):
		try:
			pos = float(self.mp.get_time()) / float(self.mp.get_length()) * 100
		except:
			pos = 0.0
		try:
			cur = str(datetime.timedelta(seconds=self.mp.get_time()/1000))[2:]
		except:
			cur = '00:00'
		try:
			tot = str(datetime.timedelta(seconds=self.mp.get_length()/1000))[2:]
		except:
			tot = '00:00'
		return [pos, cur, tot]
		
class MPlayer:
	def __init__(self, mpylayer, xid):
		if xid:
			self.mp = mpylayer.MPlayerControl(extra_args=['-af', 'volume', '-volume', '0'])
		else:
			self.mp = mpylayer.MPlayerControl(extra_args=['-vo', 'null', '-af', 'volume', '-volume', '0'])
		self.playing = False
		self.vol = 0
		self.xid = True
		
	def load(self, url):
		self.url = url
		gobject.timeout_add(1000, self.volume, self.vol)
		
	def play(self):
		if self.getPos()[1] != '00:00':
			self.mp.pause()
			self.playing = True
			return
		self.mp.loadfile(self.url)
		self.playing = True
		
	def pause(self):
		self.mp.pause()
		self.playing = False
		
	def unload(self):
		self.mp.stop()
		self.playing = False
	
	def volume(self, value):
		if value > 100.0:
			value = 100.0
		else:
			self.mp.volume = value
		self.vol = value
		if not self.mp.volume or self.mp.volume == 0.0:
			print 'retry volume'
			return True
		else:
			return False
	
	def seek(self, pos):
		try:
			goto = int(self.mp.length * float(pos / 100))
			goto = goto - self.mp.time_pos
			self.mp.seek(goto)
		except:
			pass

	def isPlaying(self):
		return self.playing
		
	def getPos(self):
		pos = self.mp.percent_pos
		if not pos:
			pos = 0.0
		try:
			cur = str(datetime.timedelta(seconds=int(self.mp.time_pos)))[2:]
		except:
			cur = '00:00'
		try:
			tot = str(datetime.timedelta(seconds=int(self.mp.length)))[2:]
		except:
			tot = '00:00'
		return [pos, cur, tot]

		
class mediaPlayer:
	def __init__(self, config, setAdjustment):
		if config.getSetting('General', 'preview_video', True):
			xid = True
		else:
			xid = False
		backend = getBackend(config.getSetting('General', 'backend', 'autodetect'), xid)
		
		if backend:
			self.backend = backend
			self.available = True
			self.load = backend.load
			self.play = backend.play
			self.pause = backend.pause
			self.unload = backend.unload
			self.volume = backend.volume
			self.seek = backend.seek
			self.isPlaying = backend.isPlaying
			self.isSeeking = False
			self.thread = None
			self.setAdjustment = setAdjustment
		else:
			self.available = False
	
	def updateAdjustment(self, ret):
		if self.thread:
			if self.thread.isAlive():
				try:
					self.thread.terminate()
				except:
					pass
		self.thread = thread2.Thread(target=self.getAdjustment, args=(self.setAdjustment, ret))
		self.thread.start()
		if not self.isPlaying():
			return False
		return ret
		
	def getAdjustment(self, callback, ret):
		pos = self.backend.getPos()
		gobject.idle_add(callback, pos, ret)
